import os
import yaml
import netCDF4
import numpy as np
import xarray as xr
import pandas as pd
from datetime import datetime, timezone
from functions import log, advanced_quality_flags

log("Performing advanced quality check")
with open("scripts/input_python.yaml", "r") as f:
    directories = yaml.load(f, Loader=yaml.FullLoader)

folder = directories["Level1_dir"]
filelist = os.listdir(directories["Level1_dir"])
filelist.sort()
filelist_all = [folder+file for file in filelist]

log("Reading Level 1 data")
dataset = xr.open_mfdataset(filelist_all)
dataset["time"] =  [datetime.timestamp(datetime.strptime(str(x.values).split(".")[0], '%Y-%m-%dT%H:%M:%S').replace(tzinfo=timezone.utc)) for x in dataset.time]

log("Apply advance quality checks to data")
advanced_dataset = advanced_quality_flags(dataset, json_path="quality_assurance.json")
dataset.close()

log("Update NetCDF files with new QA")
for file in filelist:
    file_path = os.path.join(folder, file)
    dset = netCDF4.Dataset(file_path, 'r+')
    idx = np.where((advanced_dataset["time"] >= dset["time"][0]) & (advanced_dataset["time"] <= dset["time"][-1]))[0]
    for var in dset.variables:
        if "_qual" in var:
            dset[var][:] = np.array(advanced_dataset[var][idx], dtype=bool)
    dset.close()
